﻿#region Copyright (C) 2003-2015 Stimulsoft
/*
{*******************************************************************}
{																	}
{	StimulReport.Net												}
{	Stimulsoft.Report Library										}
{																	}
{	Copyright (C) 2003-2015 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2015 Stimulsoft

using System;
using System.Windows.Forms;
using System.ComponentModel;
using Stimulsoft.Base.Localization;
using Stimulsoft.Base.Serializing;
using Stimulsoft.Report.Dictionary.Design;
using System.Data;
using System.Collections;
using Stimulsoft.Report.DotConnectOracle;
using Devart.Data.Oracle;

namespace Stimulsoft.Report.Dictionary
{
    [TypeConverter(typeof(StiSqlDatabaseConverter))]
    public class StiDotConnectOracleDatabase : StiSqlDatabase
    {
        #region StiService override
        public override string ServiceName
        {
            get
            {
                return string.Format(StiLocalization.Get("Database", "Database"), "dotConnectOracle");
            }
        }
        #endregion

        #region DataAdapter override
        protected override string DataAdapterType
        {
            get
            {
                return "Stimulsoft.Report.Dictionary.StiDotConnectOracleAdapterService";
            }
        }
        #endregion

        public override bool CanEditConnectionString
        {
            get
            {
                return true;
            }
        }

        public override string EditConnectionString(string connectionString)
        {
            DotConnectOracleConnectionEditor frm = new DotConnectOracleConnectionEditor(connectionString);
            if (frm.ShowDialog() == DialogResult.OK)
            {
                return frm.ConnectionString;
            }
            return connectionString;
        }

        //public override DialogResult Edit(bool newDatabase)
        //{
        //    using (StiSqlDatabaseEditForm form = new StiSqlDatabaseEditForm(this))
        //    {
        //        if (newDatabase) form.Text = string.Format(StiLocalization.Get("FormDatabaseEdit", "EditConnection"), "dotConnectOracle");
        //        else form.Text = string.Format(StiLocalization.Get("FormDatabaseEdit", "EditConnection"), "dotConnectOracle");

        //        form.tbName.Text = this.Name;
        //        form.tbAlias.Text = this.Alias;
        //        form.tbConnectionString.Text = this.ConnectionString;
        //        if (form.ShowDialog() == DialogResult.OK)
        //        {
        //            this.Name = form.tbName.Text;
        //            this.Alias = form.tbAlias.Text;
        //            this.ConnectionString = form.tbConnectionString.Text;
        //        }
        //        return form.DialogResult;
        //    }
        //}


        public override void ApplyDatabaseInformation(StiDatabaseInformation information, StiReport report, StiDatabaseInformation informationAll)
        {
            #region Tables

            foreach (DataTable dataTable in information.Tables)
            {
                StiDotConnectOracleSource source = new StiDotConnectOracleSource(this.Name,
                    StiNameCreation.CreateName(report, dataTable.TableName, false, false, true));
                string table = dataTable.TableName;
                if (table.Trim().Contains(" ")) table = string.Format("[{0}]", table);
                source.SqlCommand = "select * from " + table;

                foreach (DataColumn dataColumn in dataTable.Columns)
                {
                    StiDataColumn column = new StiDataColumn(dataColumn.ColumnName, dataColumn.DataType);
                    source.Columns.Add(column);
                }
                report.Dictionary.DataSources.Add(source);
            }
            #endregion

            #region Views
            foreach (DataTable dataTable in information.Views)
            {
                StiDotConnectOracleSource source = new StiDotConnectOracleSource(this.Name,
                    StiNameCreation.CreateName(report, dataTable.TableName, false, false, true));
                string table = dataTable.TableName;
                if (table.Trim().Contains(" ")) table = string.Format("[{0}]", table);
                source.SqlCommand = "select * from " + table;

                foreach (DataColumn dataColumn in dataTable.Columns)
                {
                    StiDataColumn column = new StiDataColumn(dataColumn.ColumnName, dataColumn.DataType);
                    source.Columns.Add(column);
                }
                report.Dictionary.DataSources.Add(source);
            }
            #endregion

            #region StoredProcedures
            foreach (DataTable dataTable in information.StoredProcedures)
            {
                StiDotConnectOracleSource source = new StiDotConnectOracleSource(this.Name,
                    StiNameCreation.CreateName(report, dataTable.TableName, false, false, true));
                source.SqlCommand = "execute " + dataTable.TableName;

                foreach (DataColumn dataColumn in dataTable.Columns)
                {
                    StiDataParameter parameter = new StiDataParameter();
                    parameter.Name = dataColumn.ColumnName;
                    source.Parameters.Add(parameter);
                }
                report.Dictionary.DataSources.Add(source);
            }
            #endregion
        }

        public override void ApplyDatabaseInformation(StiDatabaseInformation information, StiReport report)
        {
            ApplyDatabaseInformation(information, report, null);
        }

        /// <summary>
        /// Returns full database information.
        /// </summary>
        public override StiDatabaseInformation GetDatabaseInformation()
        {
            StiDatabaseInformation information = new StiDatabaseInformation();
            try
            {
                using (OracleConnection connection = new OracleConnection(this.ConnectionString))
                {
                    connection.Open();

                    string[] restrictions = new string[] { connection.UserId.ToUpper() };

                    #region Tables
                    Hashtable tableHash = new Hashtable();
                    try
                    {
                        DataTable tables = connection.GetSchema("Tables", restrictions);

                        string columnName = "NAME";

                        foreach (DataRow row in tables.Rows)
                        {
                            DataTable table = new DataTable(row[columnName] as string);

                            if (row["SCHEMA"] != DBNull.Value && ((string)row["SCHEMA"]) != "dbo")
                                table = new DataTable(string.Format("{1}.{0}", row["NAME"] as string, row["SCHEMA"] as string));

                            tableHash[table.TableName] = table;
                            information.Tables.Add(table);
                        }
                    }
                    catch
                    {
                    }
                    #endregion

                    #region Columns
                    try
                    {
                        DataTable columns = connection.GetSchema("Columns");

                        foreach (DataRow row in columns.Rows)
                        {
                            string columnName = row["NAME"] as string;

                            string tableName;
                            if (row["SCHEMA"] != DBNull.Value && ((string)row["SCHEMA"]) != "sys" && ((string)row["SCHEMA"]) != "dbo")
                                tableName = string.Format("{1}.{0}", row["TABLE"] as string, row["SCHEMA"] as string);
                            else
                                tableName = row["TABLE"] as string;

                            Type columnType = ConvertDbTypeToTypeInternal(row["DATATYPE"] as String);

                            DataColumn column = new DataColumn(columnName, columnType);
                            DataTable table = tableHash[tableName] as DataTable;
                            if (table != null)
                            {
                                table.Columns.Add(column);
                            }
                        }
                    }
                    catch
                    {
                    }
                    #endregion

                    connection.Close();
                }

                return information;
            }
            catch
            {
                return null;
            }
        }

        private Type ConvertDbTypeToTypeInternal(string dbType)
        {
            switch (dbType.ToLowerInvariant())
            {
                case "bigint":
                case "int":
                case "uniqueidentifier":
                case "smallint":
                case "tinyint":
                    return typeof(Int64);

                case "decimal":
                case "money":
                case "smallmoney":
                    return typeof(decimal);

                case "float":
                case "real":
                    return typeof(double);

                case "datetime":
                case "smalldatetime":
                case "timestamp":
                    return typeof(DateTime);

                default:
                    return typeof(string);
            }
        }

        /// <summary>
        /// Creates a new object of the type StiDotConnectOracleDatabase.
        /// </summary>
        public StiDotConnectOracleDatabase()
            : this(string.Empty, string.Empty)
        {
        }


        /// <summary>
        /// Creates a new object of the type StiDotConnectOracleDatabase.
        /// </summary>
        public StiDotConnectOracleDatabase(string name, string connectionString)
            : base(name, connectionString)
        {
        }


        /// <summary>
        /// Creates a new object of the type StiDotConnectOracleDatabase.
        /// </summary>
        public StiDotConnectOracleDatabase(string name, string alias, string connectionString)
            : base(name, alias, connectionString)
        {
        }


        /// <summary>
        /// Creates a new object of the type StiDotConnectOracleDatabase.
        /// </summary>
        public StiDotConnectOracleDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword)
            :
            base(name, alias, connectionString, promptUserNameAndpassword)
        {
        }
        
        /// <summary>
        /// Creates a new object of the type StiDotConnectOracleDatabase.
        /// </summary>
        public StiDotConnectOracleDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword, string key)
            :
            base(name, alias, connectionString, promptUserNameAndpassword, key)
        {
        }

    }

}